@@ -921,7 +921,9 @@ register_Matrix33()
         .def("__mul__", &mul33T<T>)
         .def("__rmul__", &rmul33T<T>)
         .def("__idiv__", &idiv33T<T>,return_internal_reference<>())
+        .def("__itruediv__", &idiv33T<T>,return_internal_reference<>())
         .def("__div__", &div33T<T>)
+        .def("__truediv__", &div33T<T>)
         .def("__add__", &add33T<T>)
         .def("__radd__", &add33T<T>)
         .def("__sub__", &subtractTL33<T>)
